#! /bin/bash 
#
# Licensed Materials - Property of IBM
# OCO Source Materials
# 
# (C) Copyright IBM Corp. 2011 All Rights Reserved
# The source code for this program is not published or otherwise divested of its 
# trade secrets, irrespective of what has been deposited with the U.S. Copyright Office.
#
# Description: This script can be used to diagnose networking, SLP, CIM 
#              communications problems on an FSM interacting with CMM, 
#              IMM, IOM, FSP.  
#
# Change Activity:
#   05/07/2007 Cramer    0000001 - initial version
#
#STARTUSAGE
#
# Usage:
#   fsmNetCheck.sh -slp all 
#   fsmNetCheck.sh -slp managed 
#   fsmNetCheck.sh -slp 9.5.3.22  
#   fsmNetCheck.sh -slp 2002:c058:6301::c058:6301
#   fsmNetCheck.sh -cmm 9.5.3.22
#   fsmNetCheck.sh -cmm managed  
#   fsmNetCheck.sh -cim 9.5.3.22
#   fsmNetCheck.sh -cim managed 
#   fsmNetCheck.sh ?
#   fsmNetCheck.sh -h | --help 
#
# Where:
#   -slp all      Do Multicast SLP search for all CMMs that will respond in current subnet 
#   -slp x.x.x.x  Do Unicast SLP seach for specific IP address target
#   -slp managed  Do Unicast SLP seach for all managed CMMs
#   -cmm x.x.x.x  SSH to target CMM and dump info about CMM 
#   -cmm managed  SSH to all managed CMMs and dump info about each CMM 
#   -cim 9.5.3.22 Do CIM calls to target CMM
#   -cim managed  Do CIM calls to all managed CMMs 
#  
#   -? | -h | --help  Prints this help 
#
# Return Codes:
# 99 - usage syntax error
# * - other system command return codes
#
#ENDUSAGE
#****************************************************************************

function showUsage() {
   # Print out the prologue comments as usage info
   sed -e '/STARTUSAGE/,/ENDUSAGE/ s/^#//' -e '1,/STARTUSAGE/ d' -e '/ENDUSAGE/,$ d' "$0"
}

#-------------------------------------------------------------------------------
# common exit point for fsmNetCheck processing
#-------------------------------------------------------------------------------
function exit_cleanup() {

   if [ $1 -eq 0 ]; then
      echo "'fsmNetCheck' command, <$cmd>, succeeded, exit code = 0." >> $log
   else
      echo "'fsmNetCheck' command, <$cmd>, failed and exited with rc=$1." >> $log
   fi

   # return the stderr from the 'fsmNetCheck' command no matter what the exit value
   if [ -f $stdErr ]; then
      cat $stdErr > /dev/stderr
   fi

   # remove temporary stdOut/stdErr files
   rm -f $stdOut
   rm -f $stdErr

   echo "fsmNetCheck.sh script end execution on `date`" >> $log
   exit $1
}


#-------------------------------------------------------------------------------
# function to get each CMM SLP repsonse and retrieve SLP attributes for each one 
#-------------------------------------------------------------------------------
function findSLPAttributes() {
# $1 is target file of SLP respomses from CMM findsrvs call  
# $2 is SLP target (CMM or IMM) 
# 
    num="`wc -l $1 | awk '{print $1}'`"
    echo "SLP Reponse count= $num"
    while read LINE 
    do     

      case "$LINE" in 
       *]*) 
          # Found an IPV6 response so extract the ip address 
          start=`expr index "$LINE" [`
                    end=`expr index "$LINE" ]`
                    len=`expr "$end" - "$start" - 1`
                     ipv6="${LINE:start:len}"
           echo "Retreiving attributes for $ipv6"
           echo "slptool -u $ipv6 findattrs $2"
           slptool -u $ipv6 findattrs $2
           echo " " 
       ;; 
       *,*) 
          # Found an IPV4 reponse 
          bs="/"
          comma=","
          start=`expr index "$LINE" "$bs"`                    
                    end=`expr index "$LINE" "$comma"`
                   len=`expr "$end" - "$start" - 2`
                   ipv4="${LINE:start+1:len}"
          echo "Retreiving attributes for $ipv4"
          echo "slptool -u $ipv4 findattrs $2"
          slptool -u $ipv4 findattrs $2
          echo " "
       ;; 
       esac 
    done <  $1 
 
    return 0
}

#-------------------------------------------------------------------------------
# function to get each IP address from an input (ifconfig) command stored in nic[] array 
#-------------------------------------------------------------------------------
function findIPAddress() {
# $1 is output of ifconfig    
 
    ipv4substring="inet addr:"
    ipv6substring="inet6 addr:"

    amp="&"

    count=0
    while read LINE 
    do     

      # extract IP4 address if it exists 
      check="${LINE/$ipv4substring/$amp}"
      start=`expr index "$check" $amp`
      if [ $start != "0" ]; then
      
          end=`expr index "$check" "B"`
          len=`expr "$end" - "$start" - 1 `
          ipv4="${check:start:len}"
          count=`expr $count + 1`
          nic[$count]=$ipv4 
      fi     


      # extract IPV6 address if it exists 
      check="${LINE/$ipv6substring/$amp}"
      start=`expr index "$check" $amp`
      if [ $start != "0" ]; then
      
          end=`expr index "$check" "/"`
          len=`expr "$end" - "$start" - 1 `
          ipv6="${check:start:len}"
          count=`expr $count + 1`
          nic[$count]=$ipv6           
      fi     


      done <  $1 
 
    nicCount=$count

}




#-------------------------------------------------------------------------------
# function to get each IP address from an input (smcli printDMData) command stored in managed[] array 
#-------------------------------------------------------------------------------
function findManagedIPAddress() {
# $1 is output of smcli printDMData
#
#  Looking for:
#
#  <chassis_ips>
# 	<ip>9.5.40.129</ip>
#   </chassis_ips>
#
    blockStart="<chassis_ips>"
    blockEnd="</chassis_ips>"
    ipStart="<ip>"
    ipEnd="</ip>"
    amp="&"
    ats="@"
    count=0
    block=0
    while read LINE 
    do     

      # extract IP4 address if it exists 
      check="${LINE/$blockStart/$amp}"

      start=`expr index "$check" $amp`
      if [ $start != "0" ]; then
          block=1
      fi 

      check="${LINE/$blockEnd/$amp}"
      start=`expr index "$check" $amp`
      if [ $start != "0" ]; then
          block=0
      fi 


      if [ $block == "1" ]; then
           check="${LINE/$ipStart/$amp}"
           start=`expr index "$check" $amp`
           check="${check/$ipEnd/$ats}"
           end=`expr index "$check" $ats`
           len=`expr "$end" - "$start" - 1 `

           if [ $start != "0" ]; then
              ipAddr="${check:start:len}"
              count=`expr $count + 1`
              managed[$count]=$ipAddr 
           fi 

      fi 
      
      done <  $1 
 
    managedCount=$count

}



function findSLPfindsrvs() {
#
#  Finds the SLP server info
#  $1 = SP Target 
#
   
   slpTarget=$1
   echo "slpTarget = $slpTarget"
   if [ "$slpTarget" = "all" ]; then
      echo "SLP multicast discovery for all CMMs in progress...."
      echo "slptool findsrvs service:management-hardware.IBM:chassis-management-module --timeout 120"
      slptool findsrvs service:management-hardware.IBM:chassis-management-module --timeout 120 > /tmp/slptool_$$.txt
      echo "SLP multicast discovery results"
      cat /tmp/slptool_$$.txt

   elif [ "$slpTarget" = "managed" ]; then

       smcli printDMData > /tmp/printDMData_$$.txt 
       findManagedIPAddress /tmp/printDMData_$$.txt 
       cnt=$managedCount 
       while [ $cnt -gt 0 ]
       do
          echo "Executing SLP discovery on managed CMM ${managed[$cnt]}"
          findSLPfindsrvs ${managed[$cnt]}
	   cnt=`expr $cnt - 1`
       done
       exit

   else
      echo "SLP unicast discovery for CMM $slpTarget in progress...."
      echo "slptool -u $slpTarget findsrvs service:management-hardware.IBM:chassis-management-module"
      slptool -u $slpTarget findsrvs service:management-hardware.IBM:chassis-management-module > /tmp/slptool_$$.txt
      echo "SLP unicast discovery results"
      cat /tmp/slptool_$$.txt
    fi 

    # Temp code to fill the text file 
    #echo "service:management-hardware.IBM:chassis-management-module://9.27.20.59,65535" > /tmp/slptool_$$.txt
    #echo "service:management-hardware.IBM:chassis-management-module://[fe80::5ef3:fcff:fe25:dd83],65535" >> /tmp/slptool_$$.txt
    #echo "service:management-hardware.IBM:chassis-management-module://9.27.20.56,65535" >> /tmp/slptool_$$.txt
    #echo "service:management-hardware.IBM:chassis-management-module://[fe80::5ef3:fcff:fe25:dccb],65535" >> /tmp/slptool_$$.txt
    #echo "service:management-hardware.IBM:chassis-management-module://9.27.20.58,65535" >> /tmp/slptool_$$.txt
    #echo "service:management-hardware.IBM:chassis-management-module://[fe80::5ef3:fcff:fe25:d7b1],65535" >> /tmp/slptool_$$.txt
    #cat /tmp/slptool_$$.txt

    findSLPAttributes /tmp/slptool_$$.txt service:management-hardware.IBM:chassis-management-module
}

function findCMMInfo() {
	#
	#  Finds the CMM info by sshing to CMM and running a number of commands 
	#  $1 = CMM Target 
	#

      if [ "$1" = "managed" ]; then

        smcli printDMData > /tmp/printDMData_$$.txt 
        findManagedIPAddress /tmp/printDMData_$$.txt 
        cnt=$managedCount 
        while [ $cnt -gt 0 ]
        do
           echo "Executing CMM SSH query on managed CMM ${managed[$cnt]}"
           findCMMInfo ${managed[$cnt]}
	    cnt=`expr $cnt - 1`
        done
        exit

      fi   


	read -p "Enter your userid for CMM $1: " userid
	echo "Please enter the CMM password when prompted"

	# System level info
	echo "baydata" > /tmp/cmm_commands_$$.txt
	echo "info" >> /tmp/cmm_commands_$$.txt
	echo "baydata" >> /tmp/cmm_commands_$$.txt
	echo "health -l a" >> /tmp/cmm_commands_$$.txt
	echo "cin" >> /tmp/cmm_commands_$$.txt
	echo "cinstatus" >> /tmp/cmm_commands_$$.txt
	echo "config" >> /tmp/cmm_commands_$$.txt
	echo "fuelg" >> /tmp/cmm_commands_$$.txt
	echo "modactlog" >> /tmp/cmm_commands_$$.txt
	echo "uplink" >> /tmp/cmm_commands_$$.txt
	echo "pmpolicy" >> /tmp/cmm_commands_$$.txt
	echo "power" >> /tmp/cmm_commands_$$.txt

	#Ping the FSM 
       ifconfig eth0 > /tmp/ipconfig_$$.txt 
       findIPAddress /tmp/ipconfig_$$.txt 
       cnt=$nicCount 
       while [ $cnt -gt 0 ]
       do
          echo "ping -i ${nic[$cnt]}" >> /tmp/cmm_commands_$$.txt
	   cnt=`expr $cnt - 1`
       done

       ifconfig eth1 > /tmp/ipconfig_$$.txt 
       findIPAddress /tmp/ipconfig_$$.txt 
       cnt=$nicCount 
       while [ $cnt -gt 0 ]
       do
          echo "ping -i ${nic[$cnt]}" >> /tmp/cmm_commands_$$.txt
	   cnt=`expr $cnt - 1`
       done


	# Chassis level info 
	echo "update -T mm[1] -a" >> /tmp/cmm_commands_$$.txt
	echo "config -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "uicfg -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "info  -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "displaylog -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "displaysd -mmstat -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "ifconfig -eth0 -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "dhcpinfo -T mm[1] -eth0" >> /tmp/cmm_commands_$$.txt
	echo "dns -T mm[1] " >> /tmp/cmm_commands_$$.txt
	echo "accseccfg -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "advfailover -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "alertcfg -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "alertentries -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "autoftp -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "clock -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "coolingzones -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "feature -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "files -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "groups -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "iocomp -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "ldapcfg -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "led -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "monalerts -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "ntp -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "permgroups -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "portcfg -T mm[1] -com1" >> /tmp/cmm_commands_$$.txt
	echo "ports -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "pwrprofile -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "remotechassis -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "security -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "snmp -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "sol -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "sshcfg -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "sslcfg -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "syslog -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "tcpcmdmode -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "temps -T mm[1]" >> /tmp/cmm_commands_$$.txt
	echo "users -T mm[1]" >> /tmp/cmm_commands_$$.txt
	
	# Switch info

	echo "info  -T switch[1]" >> /tmp/cmm_commands_$$.txt
	echo "ifconfig  -T switch[1]" >> /tmp/cmm_commands_$$.txt
	echo "fod -T switch[1]" >> /tmp/cmm_commands_$$.txt
	#
	echo "info -T switch[2]" >> /tmp/cmm_commands_$$.txt
	echo "ifconfig  -T switch[2]" >> /tmp/cmm_commands_$$.txt
	echo "fod -T switch[2]" >> /tmp/cmm_commands_$$.txt

	#Blade level info 

	
	for i in 1 2 3 4 5 6 7 8 9 10 11 12 13 14
	do
          blade="blade[$i]"
	   echo "config -T $blade" >> /tmp/cmm_commands_$$.txt
          echo "info -T $blade" >> /tmp/cmm_commands_$$.txt
          echo "ifconfig -T $blade" >> /tmp/cmm_commands_$$.txt
          echo "bootmode -T $blade" >> /tmp/cmm_commands_$$.txt
          echo "bootseq -T $blade" >> /tmp/cmm_commands_$$.txt
          echo "ethoverusb -T $blade" >> /tmp/cmm_commands_$$.txt
          echo "led -T $blade" >> /tmp/cmm_commands_$$.txt
          echo "service -T $blade" >> /tmp/cmm_commands_$$.txt
          echo "temps -T $blade" >> /tmp/cmm_commands_$$.txt
          echo "volts -T $blade" >> /tmp/cmm_commands_$$.txt
       done

       #SSH to the CMM run the commands and pipe results back
	ssh $userid@$1 < /tmp/cmm_commands_$$.txt > /tmp/cmm_results_$$.txt
	cat  /tmp/cmm_results_$$.txt

}  


function findCIMInfo() {
#
#  Finds the CIM info 
#  $1 = CIM Target 
#
#  Get CIM registered classes 

      if [ "$1" = "managed" ]; then

        smcli printDMData > /tmp/printDMData_$$.txt 
        findManagedIPAddress /tmp/printDMData_$$.txt 
        cnt=$managedCount 
        while [ $cnt -gt 0 ]
        do
           echo "Executing CIM query on managed CMM ${managed[$cnt]}"
           findCIMInfo ${managed[$cnt]}
	    cnt=`expr $cnt - 1`
        done
        exit

      fi   


   echo "List of CIM Namespaces in Registry for $1"
   echo "/opt/ibm/icc/bin/cimcli -l $1 ns"
   /opt/ibm/icc/bin/cimcli -l $1 ns
      
   echo "Enumerate all CIM instances under root/cimv2" 
   /opt/ibm/icc/bin/cimcli niall -n root/cimv2 -l $1
   echo "/opt/ibm/icc/bin/cimcli niall -n root/cimv2 -l $1"
    
}


# Initialize some constants
log='/var/hsc/log/fsmNetCheck.sh.log'
stdOut='/tmp/fsmNetCheck.sh.stdOut'
stdErr='/tmp/fsmNetCheck.sh.stdErr'

# Initialize variables that hold option values 
giveUsage=1
slpTarget=
cmmTarget=
cimTarget=
nicCount=
managed=
managedCount=


while true; do 
  case "$1" in 
    -slp | --slp | -SLP | --SLP ) 
        slpTarget=$2; 
        giveUsage=0;
        shift 2;; 
    -cmm | --cmm | -CMM | --CMM )
        cmmTarget=$2; 
        giveUsage=0;
        shift 2;; 
    -cim | --cim | -CIM | --CIM )
        cimTarget=$2; 
        giveUsage=0;
        shift 2;; 
    -h | --help | \? ) 
        giveUsage=1;
        break;;
    * ) 
        break ;; 
  esac 
  shift
done 

echo "fsmNetCheck.sh script begin execution on `date`" > $log


if [ "$giveUsage" -eq 1 ]; then
   showUsage
   exit 99
fi

if [ "$slpTarget" == "" -a "$cmmTarget" == "" -a "$cimTarget" == "" ]; then
   showUsage
   exit 99
fi

#
# Handle the various requests
#

#  First dump the FSM IP address info for eth0 and eth1
echo "FSM network config for managment network"
ifconfig eth0
echo "FSM network config for data network"
ifconfig eth1

# SLP discover processing 
if [ "$slpTarget" != "" ]; then 
   findSLPfindsrvs $slpTarget 
fi 

# CMM info processing 
if [ "$cmmTarget" != "" ]; then 
   findCMMInfo $cmmTarget  
fi    

# CIM info processing 
if [ "$cimTarget" != "" ]; then 
   findCIMInfo $cimTarget  
fi    








